@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.62.2.1 2005/03/14 19:08:33 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.67 2005/04/13 17:53:28 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -33,7 +33,6 @@
 #include "lr_parser.h"
 
 static int do_trigfunc(int s[MP_SIZE], int t[MP_SIZE]);
-static int key_equal(struct button *, struct button);
 
 static void do_accuracy();
 static void do_constant();
@@ -63,21 +62,6 @@ do_accuracy()     /* Set display accuracy. */
     }
 }
 
-
-void
-do_base(enum base_type b)    /* Change the current base setting. */
-{
-    v->base = b;
-    put_resource(R_BASE, Rbstr[(int) v->base]);
-    grey_buttons(v->base);
-    refresh_display();
-    v->pending = 0;
-    if (v->rstate) {
-        make_registers();
-    }
-}
-
-
 void
 do_business()     /* Perform special business mode calculations. */
 {
@@ -308,72 +292,29 @@ str_replace(char **str, char *from, char *to)
     }
 }
 
+
 void
-trig_filter(char **func) 
+trig_filter(char **func)
 {
-    enum mode {
-        normal = 0,
-        inv = 1,
-        hyp = 2,
-        invhyp = 3,
-    } mode;
-  
-    int inverse;
-    int hyperbolic;
+    char *tokens[4][3] = {
+        { "Sin",   "Cos",   "Tan"   },
+        { "Asin",  "Acos",  "Atan"  },
+        { "Sinh",  "Cosh",  "Tanh"  },
+        { "Asinh", "Acosh", "Atanh" },
+    };
+
+    int i;
+    int inverse    = (v->inverse)    ? 1 : 0;
+    int hyperbolic = (v->hyperbolic) ? 2 : 0;
+    int mode       = (inverse | hyperbolic);
 
     assert(func);
-    inverse = (v->inverse) ? inv : 0;
-    hyperbolic = (v->hyperbolic) ? hyp : 0;
-
-    mode = (inverse | hyperbolic);
-
-    if (!strcmp(*func, "Sin")) {
-        switch (mode) {
-            case normal:
-                break;
-            case inv:
-                str_replace(func, "Sin", "Asin");
-                break;
-            case hyp:
-                str_replace(func, "Sin", "Sinh");
-                break;
-            case invhyp:
-                str_replace(func, "Sin", "Asinh");
-                break;
-            default:
-                assert(0);
-        }
-    } else if (!strcmp(*func, "Cos")) {
-        switch (mode) {
-            case normal:
-                break;
-            case inv:
-                str_replace(func, "Cos", "Acos");
-                break;
-            case hyp:
-                str_replace(func, "Cos", "Cosh");
-                break;
-            case invhyp:
-                str_replace(func, "Cos", "Acosh");
-                break;
-            default:
-                assert(0);
-        }
-    } else if (!strcmp(*func, "Tan")) {
-        switch (mode) {
-            case normal:
-                break;
-            case inv:
-                str_replace(func, "Tan", "Atan");
-                break;
-            case hyp:
-                str_replace(func, "Tan", "Tanh");
-                break;
-            case invhyp:
-                str_replace(func, "Tan", "Atanh");
-                break;
-            default:
-                assert(0);
+  
+    if (mode) {
+        for (i = 0; i < 3; i++) {
+            if (!strcmp(*func, tokens[0][i])) {
+	        str_replace(func, tokens[0][i], tokens[mode][i]);
+            }
         }
     }
 }
@@ -422,7 +363,7 @@ do_expression()
 	int MP1[MP_SIZE];
 	char *zero = NULL;
 	do_zero(MP1);
-	zero = make_number(MP1, v->base, TRUE, FALSE);
+	zero = make_number(MP1, v->base, FALSE);
 	exp_append(zero);
       }
     }
@@ -443,11 +384,11 @@ do_expression()
 	goto out;
     } else if (v->current->flags & con) {
         int *MPval = v->MPcon_vals[char_val(v->current->value[0])];
-        exp_append(make_number(MPval, v->base, TRUE, FALSE));
+        exp_append(make_number(MPval, v->base, FALSE));
 	goto out;
     } else if (v->current->flags & bsp) {
         if (exp_has_postfix(v->expression, "Ans")) { 
-            char *ans = make_number(v->e.ans, v->base, TRUE, FALSE);   
+            char *ans = make_number(v->e.ans, v->base, FALSE);   
 
             str_replace(&v->expression, "Ans", ans);
         } 
@@ -614,7 +555,6 @@ do_trigfunc(int s[MP_SIZE], int t[MP_SIZE])
     } else assert(0);
   
     return(do_tfunc(s, t, tfunc));
-
 }
 
 
@@ -629,72 +569,25 @@ do_trig()
 int
 do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
 {
-    enum mode {
-        normal = 0,
-        inv = 1,
-        hyp = 2,
-        invhyp = 3,
-    } mode;
-
-    int inverse;
-    int hyperbolic;
-
-    if (!v->current) {
-        return(-EINVAL);
-    }
-
-    inverse = (v->inverse) ? inv : 0;
-    hyperbolic = (v->hyperbolic) ? hyp : 0;
-
-    mode = (inverse | hyperbolic);
-
-    switch (mode) {
-        case normal:
-            if (tfunc & SIN) {
-                calc_trigfunc(sin_t, s, t);
-            } else if (tfunc & COS) {
-                calc_trigfunc(cos_t, s, t);
-            } else if (tfunc & TAN) {
-                calc_trigfunc(tan_t, s, t);
-            }
-            break;
-
-        case inv:
-            if (tfunc & SIN) {
-                calc_trigfunc(asin_t, s, t);
-            } else if (tfunc & COS) {
-                calc_trigfunc(acos_t, s, t);
-            } else if (tfunc & TAN) {
-                calc_trigfunc(atan_t, s, t);
-            }
-            break;
-
-        case hyp:
-            if (tfunc & SIN) {
-                calc_trigfunc(sinh_t, s, t);
-            } else if (tfunc & COS) {
-                calc_trigfunc(cosh_t, s, t);
-            } else if (tfunc & TAN) {
-                calc_trigfunc(tanh_t, s, t);
-            }
-            break;
-
-        case invhyp:
-            if (tfunc & SIN) {
-                calc_trigfunc(asinh_t, s, t);
-            } else if (tfunc & COS) {
-                calc_trigfunc(acosh_t, s, t);
-            } else if (tfunc & TAN) {
-                calc_trigfunc(atanh_t, s, t);
-            } 
-            break;
-
-        default:
-            assert(0);
-    }
-
-
-    return(0);
+    // Assumes the SIN=0, COS=1, TAN=2
+    
+    assert(tfunc < 3); 
+    
+    enum trig_func conv_table[3][4] = {
+	{sin_t, asin_t, sinh_t, asinh_t},
+	{cos_t, acos_t, cosh_t, acosh_t},
+	{tan_t, atan_t, tanh_t, atanh_t},
+    };
+    
+    int inverse = (v->inverse) ? 1 : 0;
+    int hyperbolic = (v->hyperbolic) ? 2 : 0;
+    int mode = (inverse | hyperbolic);
+    
+    if (!v->current) return -EINVAL;
+    
+    calc_trigfunc(conv_table[tfunc][mode], s, t);
+    
+    return 0;
 }
 
 
@@ -715,6 +608,43 @@ do_clear_entry()     /* Clear the calculator display. */
     clear_display(FALSE);
 }
 
+void
+do_base(enum base_type b)    /* Change the current base setting. */
+{
+    v->base = b;
+    put_resource(R_BASE, Rbstr[(int) v->base]);
+    grey_buttons(v->base);
+    
+    switch (v->syntax) {
+    case npa:
+	v->pending = 0;
+	if (v->rstate) {
+	    make_registers();
+	}
+	break;
+	
+    case exprs: {
+	int MP[MP_SIZE];
+	int ret = usable_num(MP);
+	if (ret) {
+	    update_statusbar(_("No sane value to convert"), 
+			     "gtk-dialog-error");
+	} else {
+	    mpstr(v->e.ans, v->e.ansbak);
+	    mpstr(MP, v->e.ans);
+	    exp_replace("Ans");
+	    v->e.calc_complete = 1;
+	    make_registers();
+	}
+    }
+    break;
+    
+    default:
+	assert(0);
+    }
+    
+    refresh_display();
+}
 
 static void
 do_constant()
@@ -778,8 +708,8 @@ do_delete()     /* Remove the last numeric character typed. */
 static void
 do_exchange()         /* Exchange display with memory register. */
 {
-  int MPtemp[MP_SIZE];
-  int MPexpr[MP_SIZE];
+    int MPtemp[MP_SIZE];
+    int MPexpr[MP_SIZE];
 
     switch (v->syntax) {
         case npa:
@@ -804,6 +734,10 @@ do_exchange()         /* Exchange display with memory register. */
 	      mpstr(MPtemp, v->e.ans);	      
 	      // TODO: duplicated code in do_expression
 	      mpstr(v->e.ans, v->e.ansbak);
+	      if (v->e.expbak) {
+		free(v->e.expbak);
+		v->e.expbak = NULL;
+	      }
 	      v->e.expbak = gc_strdup(v->expression);
 	      exp_replace("Ans");
 	      v->e.calc_complete = 1;
@@ -1202,26 +1136,9 @@ void
 do_pending()
 {
 
-/*  Certain pending operations which are half completed, force the numeric
- *  keypad to be reshown (assuming they already aren't).
- *
- *  Con, Exch, Fun, Sto, Rcl and Acc    show buttons 0 - 9.
- *  < and >                             show buttons 0 - f.
- */
-
-    if (!v->ismenu) {
-        if (key_equal(v->current, KEY_CON) ||         /* Con. */
-            key_equal(v->current, KEY_EXCH) ||        /* Exch. */
-            key_equal(v->current, KEY_FUN) ||         /* Fun. */
-            key_equal(v->current, KEY_STO) ||         /* Sto. */
-            key_equal(v->current, KEY_RCL) ||         /* Rcl. */
-            key_equal(v->current, KEY_ACC)) {         /* Acc. */
-            grey_buttons(DEC);
-        }
-        if (key_equal(v->current, KEY_LSFT) ||
-            key_equal(v->current, KEY_RSFT)) {
-            grey_buttons(HEX);
-        }
+    if (!v->ismenu &&
+        !IS_KEY(v->pending, KEY_LPAR.value[0])) {
+        show_menu();
     }
 
     if (IS_KEY(v->pending, KEY_CON.value[0]))  {                 /* Con */
@@ -1259,9 +1176,6 @@ do_pending()
     }
 
     v->pending = 0;
-    if (!v->ismenu) {
-        grey_buttons(v->base);  /* Just show numeric keys for current base. */
-    }
 }
 
 
@@ -1367,7 +1281,7 @@ do_shift()     /* Perform bitwise shift on display value. */
             calc_rshift(MPval, v->e.ans, n, dir);
 
             exp_del();
-            exp_append(make_number(v->e.ans, v->base, TRUE, FALSE));
+            exp_append(make_number(v->e.ans, v->base, FALSE));
         }
         break;
 
@@ -1392,7 +1306,7 @@ do_trigtype(enum trig_type t)    /* Change the current trigonometric type. */
 }
 
 
-static int
+int
 key_equal(struct button *x, struct button y)
 {
     return(x->value[0] == y.value[0] && x->mods[0] == y.mods[0]);
