@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.39 2004/09/16 22:20:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.43 2005/03/07 22:10:16 richb Exp $
  *
- *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -314,6 +314,12 @@ read_resources()    /* Read all possible resources from the database. */
         }
     }
 
+    for (i = 0; i < MAXREGS; i++) {
+        if (get_str_resource(R_REG0 + i, str)) {
+            MPstr_to_num(str, DEC, v->MPmvals[i]);
+        }
+    }
+
     if (get_str_resource(R_BASE, str)) {
         for (i = 0; i < MAXBASES; i++) {
             if (EQUAL(str, Rbstr[i])) {
