@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/syntax_translation.c,v 1.9 2005/01/18 16:47:57 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/syntax_translation.c,v 1.11 2005/02/11 23:04:37 richb Exp $
  *
  *  Copyright (c) 2004-2005 Sami Pietila
  *
@@ -57,7 +57,7 @@ static struct word {
   { /* Bitwise NOT */                            N_("not")   },
   { /* Bitwise OR */                             N_("or")    },
   { /* PI */                                     N_("pi")    },
-  { /* Periodic payment *                        N_("pmt")   },
+  { /* Periodic payment */                       N_("pmt")   },
   { /* Present value */                          N_("pv")    },
   { /* A random number */                        N_("rand")  },
   { /* Periodic interest rate */                 N_("rate")  },
